/*
 * TComBaseLayerFilter.h
 *
 *  Created on: 2012/07/06
 *      Author: s126770
 */

#ifndef TCOMBASELAYERFILTER_H_
#define TCOMBASELAYERFILTER_H_

#include "TComPicYuv.h"

#define BLF_SIZE_MAX (8)
#define BLF_NUM_COEF_MAX (BLF_SIZE_MAX*BLF_SIZE_MAX)

struct BLFPattern {
    UInt size;
    Int  numCoef;
    Int  center_x;
    Int  center_y;
    const Int coefIdx[BLF_SIZE_MAX][BLF_SIZE_MAX]; 
};

struct BLFParam {
    Int pattern;
    UInt len;
    Int offset[3];
    Int* coef[3];
    Int* onFlag;
    Bool enabled;

    BLFParam();
    BLFParam( BLFParam& orig );
    ~BLFParam();
    BLFParam& operator=( BLFParam& orig );
};

class TComBaseLayerFilter
{
public:
  static BLFParam* initBLFParam( Int pattern=-1 );
  static const BLFPattern* getCurrentBLFPattern();

  static void findWienerFilterCoef( TComPicYuv* org, TComPicYuv* deg, BLFParam* blfp );
  static void findWienerFilterCoef( TComPicYuv* org, TComPicYuv* deg, BLFParam* blfp, Int chromaIdc );
  static void convolute( TComPicYuv* src, TComPicYuv* dst, BLFParam* blfp );
  static void convolute( TComPicYuv* src, TComPicYuv* dst, BLFParam* blfp, Int chromaIdc );

  static void calcSAD( TComPicYuv* src,TComPicYuv* dst, UInt x0, UInt y0, UInt w, UInt h, UInt64& retDist );

  static void decideBetterPicOnBlockBasis( TComPicYuv* org, TComPicYuv** srcs, Int numSrc, BLFParam* blfp );
  static void decideBetterPicOnBlockBasis( TComPicYuv* org, TComPicYuv** srcs, Int numSrc, Int* idxBest );
  static void compositeBetterPicOnBlockBasis( TComPicYuv** srcs, TComPicYuv* dst, Int* idxBest );

  static void compressChromaCoef( BLFParam* blfp );  // dCr = Cr-Cb
  static void decompressChromaCoef( BLFParam* blfp );   // Cr <- Cb+dCr

  static void printFilterFlags( Int *flags );
protected:
  static BLFParam m_blfParam;
  static void xPackCoefArray( const BLFPattern& wf, Int* coeff, Int* packedCoeff );
  static void xUnpackCoefArray( const BLFPattern& wf, Int* packedCoeff, Int* coeff );
  static Int  xQuantCoef( const BLFPattern& wf, double *coeffDbl, double adj, Int *coeff );
  static void xGetBlockRect(Int& x0, Int& y0, Int& x1, Int& y1, Int bx, Int by, Int picw, Int pich, Int chromaIdc);
  static void xPrintCoef( const BLFPattern& wf, double *coeffDbl );
  static void xPrintCoef( const BLFPattern& wf, Int *coeff, Int offset );
};

#endif /* TCOMBASELAYERFILTER_H_ */
